@@ -1,9 +1,8 @@
 package Mouse::Spec;
-
 use strict;
-use version;
+use warnings;
 
-our $VERSION = '0.33';
+our $VERSION = '0.37_01';
 
 our $MouseVersion = $VERSION;
 our $MooseVersion = '0.90';
@@ -11,6 +10,5 @@ our $MooseVersion = '0.90';
 sub MouseVersion{ $MouseVersion }
 sub MooseVersion{ $MooseVersion }
 
-
 1;
 __END__
