@@ -1,48 +1,69 @@
 package Mouse::Util;
 use strict;
 use warnings;
-use base qw/Exporter/;
+
+use Exporter;
 
 use Carp qw(confess);
 use B ();
 
+use constant _MOUSE_VERBOSE => !!$ENV{MOUSE_VERBOSE};
+
+our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(
     find_meta
     does_role
     resolve_metaclass_alias
+    apply_all_roles
     english_list
 
     load_class
     is_class_loaded
 
-    apply_all_roles
-    not_supported
-
     get_linear_isa
     get_code_info
+
+    get_code_package
+
+    not_supported
+
+    does meta dump
+    _MOUSE_VERBOSE
 );
 our %EXPORT_TAGS = (
     all  => \@EXPORT_OK,
+    meta => [qw(does meta dump _MOUSE_VERBOSE)],
 );
 
+# aliases as public APIs
+
+# it must be 'require', because Mouse::Meta::Module depends on Mouse::Util
+require Mouse::Meta::Module; # for the entities of metaclass cache utilities
+
+BEGIN {
+    *class_of                    = \&Mouse::Meta::Module::class_of;
+    *get_metaclass_by_name       = \&Mouse::Meta::Module::get_metaclass_by_name;
+    *get_all_metaclass_instances = \&Mouse::Meta::Module::get_all_metaclass_instances;
+    *get_all_metaclass_names     = \&Mouse::Meta::Module::get_all_metaclass_names;
+}
+
 # Moose::Util compatible utilities
 
 sub find_meta{
-    return Mouse::Meta::Module::class_of( $_[0] );
+    return class_of( $_[0] );
 }
 
 sub does_role{
-    my ($class_or_obj, $role) = @_;
+    my ($class_or_obj, $role_name) = @_;
 
-    my $meta = Mouse::Meta::Module::class_of($class_or_obj);
+    my $meta = class_of($class_or_obj);
 
-    return 0 unless defined $meta;
-    return 1 if $meta->does_role($role);
-    return 0;
+    (defined $role_name)
+        || ($meta || 'Mouse::Meta::Class')->throw_error("You must supply a role name to does()");
+
+    return defined($meta) && $meta->does_role($role_name);
 }
 
-
-
 BEGIN {
     my $impl;
     if ($] >= 5.009_005) {
@@ -99,6 +120,18 @@ BEGIN {
 
         return ($gv->STASH->NAME, $gv->NAME);
     }
+
+    sub get_code_package{
+        my($coderef) = @_;
+
+        my $cv = B::svref_2object($coderef);
+        $cv->isa('B::CV') or return '';
+
+        my $gv = $cv->GV;
+        $gv->isa('B::GV') or return '';
+
+        return $gv->STASH->NAME;
+    }
 }
 
 # taken from Mouse::Util (0.90)
@@ -128,6 +161,9 @@ BEGIN {
     }
 }
 
+# Utilities from Class::MOP
+
+
 # taken from Class/MOP.pm
 sub is_valid_class_name {
     my $class = shift;
@@ -145,7 +181,6 @@ sub load_first_existing_class {
     my @classes = @_
       or return;
 
-    my $found;
     my %exceptions;
     for my $class (@classes) {
         my $e = _try_load_one_class($class);
@@ -154,12 +189,11 @@ sub load_first_existing_class {
             $exceptions{$class} = $e;
         }
         else {
-            $found = $class;
-            last;
+            return $class;
         }
     }
-    return $found if $found;
 
+    # not found
     confess join(
         "\n",
         map {
@@ -170,6 +204,7 @@ sub load_first_existing_class {
 }
 
 # taken from Class/MOP.pm
+my %is_class_loaded_cache;
 sub _try_load_one_class {
     my $class = shift;
 
@@ -178,7 +213,7 @@ sub _try_load_one_class {
         confess "Invalid class name ($display)";
     }
 
-    return if is_class_loaded($class);
+    return undef if $is_class_loaded_cache{$class} ||= is_class_loaded($class);
 
     my $file = $class . '.pm';
     $file =~ s{::}{/}g;
@@ -199,14 +234,12 @@ sub load_class {
     return 1;
 }
 
-my %is_class_loaded_cache;
+
 sub is_class_loaded {
     my $class = shift;
 
     return 0 if ref($class) || !defined($class) || !length($class);
 
-    return 1 if $is_class_loaded_cache{$class};
-
     # walk the symbol table tree to avoid autovififying
     # \*{${main::}{"Foo::"}} == \*main::Foo::
 
@@ -218,15 +251,15 @@ sub is_class_loaded {
     }
 
     # check for $VERSION or @ISA
-    return ++$is_class_loaded_cache{$class} if exists $pack->{VERSION}
+    return 1 if exists $pack->{VERSION}
              && defined *{$pack->{VERSION}}{SCALAR} && defined ${ $pack->{VERSION} };
-    return ++$is_class_loaded_cache{$class} if exists $pack->{ISA}
+    return 1 if exists $pack->{ISA}
              && defined *{$pack->{ISA}}{ARRAY} && @{ $pack->{ISA} } != 0;
 
     # check for any method
     foreach my $name( keys %{$pack} ) {
         my $entry = \$pack->{$name};
-        return ++$is_class_loaded_cache{$class} if ref($entry) ne 'GLOB' || defined *{$entry}{CODE};
+        return 1 if ref($entry) ne 'GLOB' || defined *{$entry}{CODE};
     }
 
     # fail
@@ -245,7 +278,7 @@ sub apply_all_roles {
         if ($i + 1 < $max && ref($_[$i + 1])) {
             push @roles, [ $_[$i++] => $_[$i] ];
         } else {
-            push @roles, [ $_[$i] => {} ];
+            push @roles, [ $_[$i]   => undef ];
         }
         my $role_name = $roles[-1][0];
         load_class($role_name);
@@ -276,6 +309,9 @@ sub english_list {
     return join q{, }, @items, "and $tail";
 }
 
+
+# common utilities
+
 sub not_supported{
     my($feature) = @_;
 
@@ -285,19 +321,88 @@ sub not_supported{
     Carp::confess("Mouse does not currently support $feature");
 }
 
+sub meta{
+    return Mouse::Meta::Class->initialize($_[0]);
+}
+
+sub dump { 
+    my($self, $maxdepth) = @_;
+
+    require 'Data/Dumper.pm'; # we don't want to create its namespace
+    my $dd = Data::Dumper->new([$self]);
+    $dd->Maxdepth(defined($maxdepth) ? $maxdepth : 2);
+    $dd->Indent(1);
+    return $dd->Dump();
+}
+
+sub does :method;
+*does = \&does_role; # alias
+
 1;
 
 __END__
 
 =head1 NAME
 
-Mouse::Util - features, with or without their dependencies
+Mouse::Util - Features, with or without their dependencies
 
 =head1 IMPLEMENTATIONS FOR
 
-=head2 L<MRO::Compat>
+=head2 Moose::Util
 
-=head3 get_linear_isa
+=head3 C<find_meta>
+
+=head3 C<does_role>
+
+=head3 C<resolve_metaclass_alias>
+
+=head3 C<apply_all_roles>
+
+=head3 C<english_list>
+
+=head2 Class::MOP
+
+=head3 C<< is_class_loaded(ClassName) -> Bool >>
+
+Returns whether C<ClassName> is actually loaded or not. It uses a heuristic which
+involves checking for the existence of C<$VERSION>, C<@ISA>, and any
+locally-defined method.
+
+=head3 C<< load_class(ClassName) >>
+
+This will load a given C<ClassName> (or die if it is not loadable).
+This function can be used in place of tricks like
+C<eval "use $module"> or using C<require>.
+
+=head2 C<< Mouse::Util::class_of(ClassName) >>
+
+The counterpart of C<Class::MOP::class_of()>. This is not exportable.
+
+=head2 C<< Mouse::Util::get_metaclass_by_name(ClassName) >>
+
+The counterpart of C<Class::MOP::get_metaclass_by_name()>. This is not exportable.
+
+=head2 MRO::Compat
+
+=head3 C<get_linear_isa>
+
+=head2 Sub::Identify
+
+=head3 C<get_code_info>
+
+=head1 UTILITIES FOR MOUSE
+
+=head3 C<not_supported>
+
+=head1 SEE ALSO
+
+L<Moose::Util>
+
+L<Scalar::Util>
+
+L<Sub::Identify>
+
+L<MRO::Compat>
 
 =cut
 
